package com.mortals.xhx.daemon.task;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.framework.util.DateUtils;
import com.mortals.xhx.base.system.user.model.UserEntity;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.busiz.rsp.register.AppointmentDataInfo;
import com.mortals.xhx.busiz.rsp.register.AppointmentDataItem;
import com.mortals.xhx.busiz.rsp.wait.FinPersonInfo;
import com.mortals.xhx.busiz.rsp.wait.WaitPersonInfo;
import com.mortals.xhx.common.code.ProcessStatusEnum;
import com.mortals.xhx.common.code.UserType;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.RedisKey;
import com.mortals.xhx.module.appointment.model.AppointmentPersonEntity;
import com.mortals.xhx.module.appointment.model.AppointmentPersonQuery;
import com.mortals.xhx.module.appointment.service.AppointmentPersonService;
import com.mortals.xhx.module.care.model.CareConfigEntity;
import com.mortals.xhx.module.care.model.CareConfigQuery;
import com.mortals.xhx.module.care.model.CareRecordsEntity;
import com.mortals.xhx.module.care.model.CareRecordsQuery;
import com.mortals.xhx.module.care.service.CareConfigService;
import com.mortals.xhx.module.care.service.CareRecordsService;
import com.mortals.xhx.module.hik.person.model.rsp.person.PersonInfo;
import com.mortals.xhx.module.notice.model.NoticeEntity;
import com.mortals.xhx.module.notice.service.NoticeService;
import com.mortals.xhx.module.person.model.PersonEntity;
import com.mortals.xhx.module.person.service.PersonService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 同步取号等待人群与完成人群结果。
 * 通过redis list 获取
 */
@Slf4j
@Service("SyncAppointWaitAndFinTask")
public class SyncAppointWaitAndFinTaskImpl implements ITaskExcuteService {

    @Autowired
    private ICacheService cacheService;

    @Value("${php.host:''}")
    private String host;

    @Value("${php.redisDb:''}")
    private Integer redisDb;

    @Value("${spring.redis.database:''}")
    private Integer database;

    @Autowired
    private CareRecordsService careRecordsService;
    @Autowired
    private CareConfigService careConfigService;

    @Override
    public void excuteTask(ITask task) throws AppException {
        log.debug("同步今天等待用户任务");
        syncWaitAndFinQueue();
        log.debug("同步今天等待用户完成任务");
        log.debug("关怀用户任务");
        updateWaitQueueTime();
        log.debug("关怀服务完成任务");
    }

    private void syncWaitAndFinQueue() {
        cacheService.select(redisDb);//
        List<WaitPersonInfo> waitPersonInfos = cacheService.lrange(RedisKey.KEY_WAITNUM_LIST_CACHE, WaitPersonInfo.class);
        List<FinPersonInfo> finPersonInfos = cacheService.lrange(RedisKey.KEY_FIN_LIST_CACHE, FinPersonInfo.class);
        cacheService.select(database);//
        if (!ObjectUtils.isEmpty(waitPersonInfos)) {
            List<CareRecordsEntity> recordsEntityList = waitPersonInfos.stream().map(item -> {
                CareRecordsEntity careRecordsEntity = new CareRecordsEntity();
                careRecordsEntity.initAttrValue();
                careRecordsEntity.setName(item.getIdcardName());
                careRecordsEntity.setContact(item.getPhone());
                careRecordsEntity.setIdCard(item.getIdcardIDCardNo());
                careRecordsEntity.setBussinessId(item.getBusinessId());
                careRecordsEntity.setBussinessName(item.getBusinessName());
                careRecordsEntity.setService(item.getBusinessName());
                careRecordsEntity.setQueueNo(item.getFlowNum());
                careRecordsEntity.setMonitorTime(new Date());
                careRecordsEntity.setTakeTime(item.getTakeTime() == null ? new Date() : item.getTakeTime());
                careRecordsEntity.setPersonId(item.getWaitId());
                careRecordsEntity.setProcessStatus(ProcessStatusEnum.排队中.getValue());
                careRecordsEntity.setCreateTime(new Date());
                careRecordsEntity.setCreateUserName("system");
                careRecordsEntity.setCreateUserId(1L);
                return careRecordsEntity;
            }).collect(Collectors.toList());

            if (!ObjectUtils.isEmpty(recordsEntityList)) {
                log.info("保存关怀记录数量，size:{}", recordsEntityList.size());
                careRecordsService.save(recordsEntityList);
            }

            if (!ObjectUtils.isEmpty(finPersonInfos)) {
                List<Long> waitIdList = finPersonInfos.stream().map(item -> item.getWaitId()).collect(Collectors.toList());
                Map<Long, FinPersonInfo> waitMap = finPersonInfos.stream().collect(Collectors.toMap(x -> x.getWaitId(), y -> y, (o, n) -> n));

                CareRecordsQuery careRecordsQuery = new CareRecordsQuery();
                careRecordsQuery.setPersonIdList(waitIdList);
                List<CareRecordsEntity> careRecordsEntities = careRecordsService.find(careRecordsQuery);
                if (!ObjectUtils.isEmpty(careRecordsEntities)) {
                    //todo 更新关怀记录
                    careRecordsEntities.forEach(item -> {
                        FinPersonInfo finPersonInfo = waitMap.get(item.getPersonId());
                        item.setProcessStatus(ProcessStatusEnum.接件结束.getValue());
                        item.setCallTime(finPersonInfo.getCalltime());
                        Long diff = DateUtil.between(item.getTakeTime(), finPersonInfo.getCalltime(), DateUnit.MINUTE);
                        item.setWaitTime(diff.intValue());
                        item.setUpdateTime(new Date());
                        item.setUpdateUserId(1L);
                        item.setUpdateUserName("system");
                    });
                    careRecordsService.update(careRecordsEntities);
                }
            }
        }
    }

    private void updateWaitQueueTime() {
        CareConfigEntity careConfigEntity = careConfigService.selectOne(new CareConfigQuery());
        List<CareRecordsEntity> careRecordsEntities = careRecordsService.find(new CareRecordsQuery().processStatus(ProcessStatusEnum.排队中.getValue()));
        if (!ObjectUtils.isEmpty(careRecordsEntities)) {
            //更新等待时长
            List<CareRecordsEntity> recordsEntities = careRecordsEntities.stream().map(item -> {
                Long diff = DateUtil.between(item.getTakeTime(), new Date(), DateUnit.MINUTE);
                item.setWaitTime(diff.intValue());
                item.setUpdateTime(new Date());
                item.setUpdateUserId(1L);
                item.setUpdateUserName("system");
                return item;
            }).collect(Collectors.toList());
            careRecordsService.update(recordsEntities);

            //查询关怀配置，发送管理员通知消息
            if (!ObjectUtils.isEmpty(careConfigEntity)) {
                if (checkCareServiceUsed(careConfigEntity)) return;
                //校验当天日期是否启用
                if (checkCareServiceDay(careConfigEntity)) return;


                Long serviceThreshold = careConfigEntity.getServiceThreshold();
                String msgRecipients = careConfigEntity.getMsgRecipients();

                List<CareRecordsEntity> waitCareList = recordsEntities.stream().filter(f -> f.getWaitTime() > serviceThreshold.intValue() && f.getCareCount() == 0).collect(Collectors.toList());
                if (!ObjectUtils.isEmpty(waitCareList)) {
                    waitCareList.forEach(waitCare -> {
                        //发送消息给管理员
                        if (StrUtil.contains(msgRecipients, "管理员")) {
                            careRecordsService.notice(waitCare, null);
                        }
                        //发送短信消息给办事群众
                        if (StrUtil.contains(msgRecipients, "办事群众")) {
                            careRecordsService.toCare(waitCare, null);
                        }
                    });
                }
            }

        }
    }

    private boolean checkCareServiceUsed(CareConfigEntity careConfigEntity) {
        Integer used = careConfigEntity.getUsed();
        if (used == YesNoEnum.NO.getValue()) {
            log.info("关怀服务未启用！");
            return true;
        }
        return false;
    }

    private boolean checkCareServiceDay(CareConfigEntity careConfigEntity) {
        switch (DateTime.now().dayOfWeekEnum().getValue()) {
            case 1:
                if (careConfigEntity.getMonday() == 1) return true;
            case 2:
                if (careConfigEntity.getThursday() == 1) return true;
            case 3:
                if (careConfigEntity.getWednesday() == 1) return true;
            case 4:
                if (careConfigEntity.getThursday() == 1) return true;
            case 5:
                if (careConfigEntity.getFriday() == 1) return true;
            case 6:
                if (careConfigEntity.getSaturday() == 1) return true;
            case 7:
                if (careConfigEntity.getSunday() == 1) return true;
            default:
                return false;
        }
    }

    @Override
    public void stopTask(ITask task) throws AppException {

    }


    public static void main(String[] args) {


        System.out.println(DateUtil.parse("19951071", "yyyyMMdd"));
    }


}
